@@ -8,11 +8,43 @@ need to be stored, and as a class with a __call__ method if there are parameters
 import re
 from .qualtrim import quality_trim_index, nextseq_trim_index
 from .align import Aligner, SEMIGLOBAL, InsertAligner
-from .util import reverse_complement
+from .util import complement, reverse_complement
 from collections import defaultdict
 import copy
 import logging
 
+# Base classes
+
+class ReadPairModifier(object):
+    """Superclass of modifiers that edit a pair of reads simultaneously."""
+    def __call__(self, read1, read2):
+        raise NotImplemented()
+
+class Trimmer(object):
+    def __init__(self):
+        self.trimmed_bases = 0
+    
+    def __call__(self, read):
+        raise NotImplemented()
+    
+    def subseq(self, read, begin=0, end=None):
+        if begin or (end is not None):
+            front_bases, back_bases, new_read = read.subseq(begin, end)
+            self.trimmed_bases += front_bases + back_bases
+            return new_read
+        else:
+            return read
+    
+    def clip(self, read, front=0, back=0):
+        if front or back:
+            front_bases, back_bases, new_read = read.clip(front, back)
+            self.trimmed_bases += front_bases + back_bases
+            return new_read
+        else:
+            return read
+
+# Modifiers
+
 class AdapterCutter(object):
     """
     Repeatedly find one of multiple adapters in reads.
@@ -110,50 +142,54 @@ class AdapterCutter(object):
         self.with_adapters += 1
         return trimmed_read
 
-class ReadPairModifier(object):
-    """Superclass of modifiers that edit a pair of reads simultaneously."""
-    def __call__(self, read1, read2):
-        raise NotImplemented()
-
 class InsertAdapterCutter(ReadPairModifier):
     """
     AdapterCutter that uses InsertAligner to first try to identify
     insert overlap before falling back to semi-global adapter alignment.
     """
-    def __init__(self, adapter1, adapter2, action='trim', symmetric=True, **aligner_args):
+    def __init__(self, adapter1, adapter2, action='trim', mismatch_action=None, symmetric=True, **aligner_args):
         self.adapter1 = adapter1
         self.adapter2 = adapter2
         self.aligner = InsertAligner(adapter1.sequence, adapter2.sequence, **aligner_args)
         self.action = action
+        self.mismatch_action = mismatch_action
         self.symmetric = symmetric
         self.with_adapters = [0, 0]
+        self.corrected_pairs = 0
+        self.corrected_bp = [0, 0]
     
     def __call__(self, read1, read2):
-        match1, match2, insert_match = self.aligner.match_insert(read1.sequence, read2.sequence)
+        insert_match, insert_match_len, adapter_match1, adapter_match2 = self.aligner.match_insert(
+            read1.sequence, read2.sequence)
         
-        if match1 is None and match2 is None:
-            match1 = self.adapter1.match_to(read1)
-            match2 = self.adapter2.match_to(read2)
+        if (self.mismatch_action and insert_match and insert_match[5] > 0 and
+                (adapter_match1 or adapter_match2)):
+            self.correct_errors(read1, read2, insert_match)
+        
+        if adapter_match1 is None and adapter_match2 is None:
+            adapter_match1 = self.adapter1.match_to(read1)
+            adapter_match2 = self.adapter2.match_to(read2)
         
         if self.symmetric:
-            if match1 is None and match2:
-                match1 = match2.copy()
-            if match2 is None and match1:
-                match2 = match1.copy()
+            if adapter_match1 is None and adapter_match2:
+                adapter_match1 = adapter_match2.copy()
+            if adapter_match2 is None and adapter_match1:
+                adapter_match2 = adapter_match1.copy()
             
-        if match1 is not None:
-            match1.adapter = self.adapter1
-            match1.read = read1
-            match1.front = False
+        if adapter_match1 is not None:
+            adapter_match1.adapter = self.adapter1
+            adapter_match1.read = read1
+            adapter_match1.front = False
         
-        if match2 is not None:
-            match2.adapter = self.adapter2
-            match2.read = read2
-            match2.front = False
+        if adapter_match2 is not None:
+            adapter_match2.adapter = self.adapter2
+            adapter_match2.read = read2
+            adapter_match2.front = False
         
-        return (self.trim(read1, match1, 0), self.trim(read2, match2, 1))
+        return (self.trim(read1, adapter_match1, 0), self.trim(read2, adapter_match2, 1))
     
     def trim(self, read, match, read_idx):
+        # TODO: refactor so that both adapter cutters use the same function
         if not match:
             read.match = None
             read.match_info = None
@@ -182,29 +218,72 @@ class InsertAdapterCutter(ReadPairModifier):
         
         self.with_adapters[read_idx] += 1
         return trimmed_read
-
-class Trimmer(object):
-    def __init__(self):
-        self.trimmed_bases = 0
     
-    def __call__(self, read):
-        raise NotImplemented()
-    
-    def subseq(self, read, begin=0, end=None):
-        if begin or (end is not None):
-            front_bases, back_bases, new_read = read.subseq(begin, end)
-            self.trimmed_bases += front_bases + back_bases
-            return new_read
-        else:
-            return read
-    
-    def clip(self, read, front=0, back=0):
-        if front or back:
-            front_bases, back_bases, new_read = read.clip(front, back)
-            self.trimmed_bases += front_bases + back_bases
-            return new_read
-        else:
-            return read
+    def correct_errors(self, read1, read2, insert_match):
+        # read2 reverse-complement is the reference, read1 is the query
+        r1_seq = list(read1.sequence)
+        r2_seq = list(read2.sequence)
+        l2 = len(r2_seq)
+        
+        has_quals = read1.qualities and read2.qualities
+        if has_quals:
+            r1_qual = list(read1.qualities)
+            r2_qual = list(read2.qualities)
+        elif self.mismatch_action == 'best':
+            raise Exception("Cannot perform quality-based error correction on reads lacking quality information")
+        
+        r1_changed = 0
+        r2_changed = 0
+        
+        for i, j in zip(
+                range(insert_match[2], insert_match[3]),
+                range(l2 - insert_match[0] - 1, l2 - insert_match[1] - 1, -1)):
+            b1 = r1_seq[i]
+            b2 = complement[r2_seq[j]]
+            if b1 == b2:
+                continue
+            if b1 == 'N':
+                r1_seq[i] = b2
+                if has_quals:
+                    r1_qual[i] = r2_qual[j]
+                r1_changed += 1
+            elif b2 == 'N':
+                r2_seq[j] = complement[b1]
+                if has_quals:
+                    r2_qual[j] = r1_qual[i]
+                r2_changed += 1
+            elif self.mismatch_action == 'N':
+                r1_seq[i] = 'N'
+                r2_seq[i] = 'N'
+                r1_changed += 1
+                r2_changed += 1
+            elif r1_qual[i] == r2_qual[j]:
+                # default to read1 base
+                r2_seq[j] = complement[b1]
+                r2_changed += 1
+            elif r1_qual[i] > r2_qual[j]:
+                r2_seq[j] = complement[b1]
+                r2_qual[j] = r1_qual[i]
+                r2_changed += 1
+            else:
+                r1_seq[i] = b2
+                r1_qual[i] = r2_qual[j]
+                r1_changed += 1
+        
+        if r1_changed or r2_changed:
+            self.corrected_pairs += 1
+            if r1_changed:
+                self.corrected_bp[0] += r1_changed
+                read1.sequence = ''.join(r1_seq)
+                read1.corrected = r1_changed
+                if has_quals:
+                    read1.qualities = ''.join(r1_qual)
+            if r2_changed:
+                self.corrected_bp[1] += r2_changed
+                read2.sequence = ''.join(r2_seq)
+                read2.corrected = r2_changed
+                if has_quals:
+                    read2.qualities = ''.join(r2_qual)
 
 class UnconditionalCutter(Trimmer):
     """
@@ -255,6 +334,10 @@ class MinCutter(Trimmer):
             else:
                 return read
         
+        # TODO: distinguish between adapter trimming and other trimming.
+        # For things like Methyl-Seq, we want to trim additional bases
+        # after adapter trimming, but we want to count other post-adapter
+        # trimmed bases toward the minimum length
         def to_trim(offset, is_front):
             if self.count_trimmed:
                 trimmed = read.clipped[offset] + read.clipped[offset+2]
@@ -398,75 +481,79 @@ class NEndTrimmer(Trimmer):
         end_cut = end_cut.start() if end_cut else len(read)
         return self.subseq(read, start_cut, end_cut)
 
-class BisulfiteTrimmer(MinCutter):
+class RRBSTrimmer(MinCutter):
     """
-    Generic bisulfite trimmer. Sequences that are adapter-trimmed are further
-    trimmed 2 bp on the 3' end to remove potential methylation-biased bases
-    from the end-repair reaction.
+    Sequences that are adapter-trimmed are further trimmed 2 bp on the 3' end to
+    remove potential methylation-biased bases from the end-repair reaction.
     """
     
-    display_str = "Bisulfite-trimmed"
+    display_str = "RRBS-trimmed"
     
     def __init__(self, trim_5p=0, trim_3p=2):
-        super(BisulfiteTrimmer, self).__init__((trim_5p, -1 * trim_3p), count_trimmed=False, only_trimmed=True)
+        super(RRBSTrimmer, self).__init__((trim_5p, -1 * trim_3p), count_trimmed=False, only_trimmed=True)
 
 class NonDirectionalBisulfiteTrimmer(object):
     """
     For non-directional RRBS/WGBS libraries (which implies that they were digested
-    using MspI), sequences starting with either 'CAA' or 'CGA' will have 2 bp
-    trimmed off either end to remove potential methylation-biased bases from the
-    end-repair reaction. Otherwise only the 3' end of adapter-trimmed reads are
-    further trimmed 2 bp.
+    using MspI), sequences that start with either 'CAA' or 'CGA' will have 2 bp
+    trimmed off the 5' end to remove potential methylation-biased bases from the
+    end-repair reaction. Additionally, for RRBS reads, if CAA/CGA is not trimmed
+    *and* the read has been adapter-trimmed, a minimum number of bases is trimmed
+    from the 3' end.
     """
     
     display_str = "Bisulfite-trimmed (Non-directional)"
     _regex = re.compile("^C[AG]A")
     
-    def __init__(self, trim_5p=2, trim_3p=2):
-        self._3pCutter = MinCutter((-1 * trim_3p,), count_trimmed=False, only_trimmed=True)
-        self._bothCutter = MinCutter((trim_5p, -1 * trim_3p), count_trimmed=False)
+    def __init__(self, trim_5p=2, trim_3p=2, rrbs=False):
+        self._non_directional_cutter = MinCutter([trim_5p], count_trimmed=False, only_trimmed=False)
+        self.rrbs = rrbs
+        if rrbs:
+            self._rrbs_cutter = RRBSTrimmer(trim_3p)
     
     def __call__(self, read):
         cutter = None
         if self._regex.match(read.sequence):
-            cutter = self._bothCutter
-        elif read.match:
-            cutter = self._3pCutter
+            cutter = self._non_directional_cutter
+        elif self.rrbs:
+            cutter = self._rrbs_cutter
         return cutter(read) if cutter else read
     
     @property
     def modified_bases(self):
-        return self._3pCutter.trimmed_bases + self._bothCutter.trimmed_bases
+        return self._rrbs_cutter.trimmed_bases + self._non_directional_cutter.trimmed_bases
 
-class EpiGnomeBisulfiteTrimmer(MinCutter):
-    """EpiGnome reads are trimmed 6 bp on the 5' end."""
+class TruSeqBisulfiteTrimmer(MinCutter):
+    """EpiGnome reads are trimmed at least 6 bp on the 5' end."""
     
-    display_str = "Bisulfite-trimmed (EpiGnome)"
+    display_str = "Bisulfite-trimmed (EpiGnome/TruSeq)"
     
     def __init__(self):
-        super(EpiGnomeBisulfiteTrimmer, self).__init__((6,), count_trimmed=True)
+        super(TruSeqBisulfiteTrimmer, self).__init__((6,), count_trimmed=True, only_trimmed=False)
 
-class SwiftBisulfiteTrimmer(object):
+class SwiftBisulfiteTrimmer(ReadPairModifier):
     """
-    For WGBS libraries prepared with the Swift Accel-NGS kit, 10 bp are trimmed off the 5' end.
-    Additionally, if the read length is longer than 100 bp, 10 bp ar trimmed off the 3' end.
+    For WGBS libraries prepared with the Swift Accel-NGS kit, 10 bp are trimmed off the end
+    of read1 and the beginning of read2.
     """
     
     display_str = "Bisulfite-trimmed (Swift)"
     
-    def __init__(self):
-        self._shortCutter = MinCutter((10,), count_trimmed=False, only_trimmed=False)
-        self._longCutter = MinCutter((10,-10), count_trimmed=False, only_trimmed=False)
+    def __init__(self, trim_5p1=0, trim_3p1=10, trim_5p2=10, trim_3p2=0):
+        self._read1_cutter = MinCutter((trim_5p1, -1 * trim_3p1), count_trimmed=False, only_trimmed=False)
+        self._read2_cutter = MinCutter((trim_5p2, -1 * trim_3p2), count_trimmed=False, only_trimmed=False)
     
-    def __call__(self, read):
-        cutter = self._longCutter if read.original_length > 100 else self._shortCutter
-        return cutter(read)
+    def __call__(self, read1, read2):
+        return (self._read1_cutter(read1), self._read2_cutter(read2))
 
     @property
     def modified_bases(self):
-        return self._shortCutter.trimmed_bases + self._longCutter.trimmed_bases
+        return self._read1_cutter.trimmed_bases + self._read2_cutter.trimmed_bases
 
-class MergeOverlapping(object):
+# TODO: InsertAdapterCutter should save the insert match, and
+# MergeOverlapping should use that rather than doing another alignment
+
+class MergeOverlapping(ReadPairModifier):
     def __init__(self, min_overlap=0.9, error_rate=0.1):
         self.min_overlap = int(min_overlap) if min_overlap > 1 else min_overlap
         self.error_rate = error_rate
@@ -515,11 +602,10 @@ class MergeOverlapping(object):
         return (read1, read2)
 
 class Modifiers(object):
-    def __init__(self, paired, merger=None):
+    def __init__(self, paired):
         self.modifiers = []
         self.modifier_indexes = defaultdict(lambda: [])
         self.paired = paired
-        self.merger = merger
     
     def add_modifier(self, mod_class, read=1|2, **kwargs):
         if issubclass(mod_class, ReadPairModifier):
@@ -580,8 +666,6 @@ class Modifiers(object):
                         read1 = mods[0](read1)
                     if mods[1] is not None:
                         read2 = mods[1](read2)
-            if self.merger:
-                read1, read2 = self.merger.merge(read1, read2)
             reads = [read1, read2]
         else:
             read = record
