@@ -167,7 +167,7 @@ class WorkerProcess(Process):
                 process_stats = collect_process_statistics(
                     self.processed_reads, self.total_bp1, self.total_bp2,
                     self.modifiers, self.filters, self.formatters)
-                adapter_stats = summarize_adapters(self.modifiers.get_modifier_pair(AdapterCutter))
+                adapter_stats = summarize_adapters(self.modifiers.get_modifiers(AdapterCutter))
             enqueue(
                 self.summary_queue,
                 (self.index, self.seen_batches, process_stats, adapter_stats),
@@ -643,7 +643,7 @@ def run_parallel(reader, modifiers, filters, formatters, writers, threads=2, tim
             raise
 
     except (FormatError, EOFError) as e:
-        logging.getLogger().error("Cutadapt error", exc_info=True)
+        logging.getLogger().error("Atropos error", exc_info=True)
         rc = 1
 
     except Exception as e:
