@@ -46,6 +46,7 @@ from atropos.adapters import AdapterParser, BACK
 from atropos.modifiers import *
 from atropos.filters import *
 from atropos.report import *
+from atropos.util import int_or_str
 
 import argparse
 from collections import defaultdict
@@ -83,12 +84,11 @@ def main(cmdlineargs=None, default_outfile="-"):
     
     # dispatch to subcommands if one is specified
     if options.command == "detect":
-        #try:
         import atropos.detect
         atropos.detect.main(options)
-        #except:
-        #    parser.error("Error during adapter detection; do you have khmer installed?")
-        #    sys.exit(1)
+    elif options.command == "error":
+        import atropos.error
+        atropos.error.main(options)
     # otherwise trim reads
     else:
         run_atropos(options, parser, default_outfile="-")
@@ -181,7 +181,7 @@ def get_argument_parser():
         help="Input file format; can be either 'fasta', 'fastq' or 'sra-fastq'. "
             "Ignored when reading csfasta/qual files. (auto-detect "
             "from file name extension)")
-    group.add_argument("--max-reads", default=None,
+    group.add_argument("--max-reads", type=int_or_str, default=None,
         help="Maximum number of reads/pairs to process (no max)")
     
     group = parser.add_argument_group("Finding adapters",
@@ -211,6 +211,14 @@ def get_argument_parser():
             "as with -a. This option is mostly for rescuing failed library "
             "preparations - do not use if you know which end your adapter was "
             "ligated to! (none)")
+    group.add_argument("--no-trim", dest='action', action='store_const', const=None,
+        help="Match and redirect reads to output/untrimmed-output as usual, "
+            "but do not remove adapters. (no)")
+    group.add_argument("--mask-adapter", dest='action', action='store_const', const='mask',
+        help="Mask adapters with 'N' characters instead of trimming them. (no)")
+    
+    ## Arguments specific to the choice of aligner
+    
     group.add_argument("--aligner", choices=('adapter', 'insert'), default='adapter',
         help="Which alignment algorithm to use for identifying adapters. Currently, "
              "you can choose between the semi-global alignment strategy used in Cutdapt "
@@ -218,8 +226,10 @@ def get_argument_parser():
              "Note that insert-based alignment can only be used with paired-end reads "
              "containing 3' adapters. New algorithms are being implemented and the default "
              "is likely to change. (adapter)")
+    
+    # Arguments for adapter match
     group.add_argument("-e", "--error-rate", type=float, default=None,
-        help="Maximum allowed error rate (no. of errors divided by the length "
+        help="Maximum allowed error rate for adapter match (no. of errors divided by the length "
             "of the matching region). (0.1)")
     group.add_argument("--no-indels", action='store_false', dest='indels', default=True,
         help="Allow only mismatches in alignments. "
@@ -235,13 +245,22 @@ def get_argument_parser():
     group.add_argument("-N", "--no-match-adapter-wildcards", action="store_false",
         default=True, dest='match_adapter_wildcards',
         help="Do not interpret IUPAC wildcards in adapters. (no)")
-    group.add_argument("--no-trim", dest='action', action='store_const', const=None,
-        help="Match and redirect reads to output/untrimmed-output as usual, "
-            "but do not remove adapters. (no)")
-    group.add_argument("--mask-adapter", dest='action', action='store_const', const='mask',
-        help="Mask adapters with 'N' characters instead of trimming them. (no)")
+    
+    # Arguments for insert match
+    group.add_argument("--insert-match-error-rate", type=float, default=None,
+        help="Maximum allowed error rate for insert match (no. of errors divided by the length "
+            "of the matching region). (0.2)")
+    group.add_argument("--insert-match-adapter-error-rate", type=float, default=None,
+        help="Maximum allowed error rate for matching adapters after successful insert match "
+             "(no. of errors divided by the length of the matching region). (0.2)")
+    group.add_argument("--correct-mismatches", choices=("best", "N"), default=None,
+        help="How to handle mismatches while aligning/merging; best: select the base with the best "
+             "quality; N: set the base to N. Note that if exactly one base is ambiguous, the "
+             "non-ambiguous base is always used.")
+    
+    # Merging arguments
     group.add_argument("-R", "--merge-overlapping", action="store_true", default=False,
-        help="Merge read pairs that overlap into a single sequence. This is an experimental "
+        help="Merge read pairs that overlap into a single sequence. This is experimental "
              "and currently only works with read pairs where an adapter match was found at "
              "the 3' ends of both reads. The minimum required overlap is controled by "
              "--merge-min-overlap, and the mimimum error rate in the alignment is controlled "
@@ -251,11 +270,6 @@ def get_argument_parser():
              "it specifies the minimum length as the fraction of the length of the *shorter* read "
              "in the pair; otherwise it specifies the minimum number of overlapping base pairs ("
              "with an absolute minimum of 2 bp). (0.9)")
-    # TODO: error correction:
-    #group.add_argument("--merge-mismatches", choices=("best", "N", "random"), default="best",
-    #	help="How to handle mismatches while merging; best: select the base with the best quality; "
-    #		 "N: set the base to N; random: select one of the two bases at random. Note that if "
-    #		 "exactly one base is ambiguous, the non-ambiguous base is always used.")
     
     group = parser.add_argument_group("Additional read modifications")
     group.add_argument("-u", "--cut", action='append', default=[], type=int, metavar="LENGTH",
@@ -424,12 +438,13 @@ def get_argument_parser():
     group = parser.add_argument_group("Method-specific options")
     group.add_argument("--bisulfite", default=False, metavar="METHOD",
         help="Set default option values for bisulfite-treated data. The argument specifies the "
-             "type of bisulfite library (non-directional, truseq, epignome, or swift) or custom "
-             "parameters for trimming: '<read1>[;<read2>]' where trimming parameters for each "
-             "read are: '<5' cut>,<3' cut>,<include trimmed>,<only trimmed>' where 'include "
-             "trimmed' is 1 or 0 for whether or not the bases already trimmed during/prior to adapter "
-             "trimming should be counted towards the total bases to be cut and 'only trimmed' "
-             "is 1 or 0 for whether or not only trimmed reads should be further cut. (no)")
+             "type of bisulfite library (rrbs, non-directional, non-directional-rrbs, truseq, "
+             "epignome, or swift) or custom parameters for trimming: '<read1>[;<read2>]' where "
+             "trimming parameters for each read are: '<5' cut>,<3' cut>,<include trimmed>,<only trimmed>' "
+             "where 'include trimmed' is 1 or 0 for whether or not the bases already trimmed "
+             "during/prior to adapter trimming should be counted towards the total bases to be "
+             "cut and 'only trimmed' is 1 or 0 for whether or not only trimmed reads should be "
+             "further cut. (no)")
     group.add_argument("--mirna", action="store_true", default=False,
         help="Set default option values for miRNA data. (no)")
     
@@ -464,15 +479,32 @@ def get_argument_parser():
         help='sub-command help')
     
     detect_parser = subparsers.add_parser("detect", help="Detect adapter sequences",
-        usage="\natropos -se input.fastq detect\natropos -pe1 in1.fastq -pe2 in2.fastq detect",
+        usage="\natropos -se input.fastq detect\natropos -pe1 in1.fq -pe2 in2.fq detect",
         description="Detect adapter sequences directly from read sequences.")
-    detect_parser.set_defaults(command='detect', max_reads=10000)
+    detect_parser.set_defaults(command='detect')
     detect_parser.add_argument("--kmer-size", type=int, default=12,
         help="Size of k-mer used to scan reads for adapter sequences. (12)")
     detect_parser.add_argument("--max-adapters", type=int, default=None,
-        help="The maximum number of candidate adapters to report. (10000)")
+        help="The maximum number of candidate adapters to report. (none)")
+    detect_parser.add_argument("--include-unknown", action="store_true", default=False,
+        help="Also report sequences that are not in the list of known adapters.")
     detect_parser.add_argument("--adapter-report", default="-",
         help="File in which to write the summary of detected adapters. (stdout)")
+    detect_parser.add_argument("--known-contaminant", action="append", default=None,
+        help="Pass known contaminants in on the commandline as 'name=sequence'. "
+             "Can be specified multiple times.")
+    detect_parser.add_argument("--known-contaminants-file", default=None,
+        help="File with known contaminants, one per line, with name and sequence "
+             "separated by one or more tabs.")
+    
+    error_parser = subparsers.add_parser("error", help="Estimate empirical error rate",
+        usage="\natropos -se input.fastq error\natropos -pe in1.fq -pe2 in2.fq error",
+        description="Estimate the error rate from base qualities. This can help to "
+                    "determine the quality of your data, and to decide the value for "
+                    "the max error rate (-e) parameter. Normal error for an Illumina "
+                    "experiment is around 1% or less. We recommend setting -e to "
+                    "10 x the empirical error rate.")
+    error_parser.set_defaults(command='error')
     
     return parser
 
@@ -489,12 +521,6 @@ def setup_logging(stdout=False, level="INFO"):
     logging.getLogger().setLevel(level)
     logging.getLogger().addHandler(stream_handler)
 
-MAGNITUDE = dict(
-    G=(1E9, "000000000"),
-    M=(1E6, "000000"),
-    K=(1E3, "000")
-)
-
 def validate_options(options, parser):
     if sum(int(opt is not False) for opt in (options.mirna, options.bisulfite)) > 1:
         parser.error("Cannot specify more than one method-specific option")
@@ -549,10 +575,15 @@ def validate_options(options, parser):
             paired = 'both'
 
     if options.aligner != 'adapter':
-        if options.aligner == 'insert' and paired != 'both':
-            parser.error("Insert aligner only works with paired-end reads")
-            # TODO: should also be checking that there is exactly one 3' adapter for each read
-            # TODO: have the aligner tell us whether it can be used based on options?
+        if options.aligner == 'insert':
+            if paired != 'both':
+                parser.error("Insert aligner only works with paired-end reads")
+                # TODO: should also be checking that there is exactly one 3' adapter for each read
+                # TODO: have the aligner tell us whether it can be used based on options?
+            if options.insert_match_error_rate is None:
+                options.insert_match_error_rate = options.error_rate or 0.2
+            if options.insert_match_adapter_error_rate is None:
+                options.insert_match_adapter_error_rate = options.error_rate or 0.2
     
     if options.merge_overlapping and (
             paired != "both" or
@@ -581,9 +612,14 @@ def validate_options(options, parser):
         if options.error_rate is None:
             options.error_rate = 0.12
     elif options.bisulfite:
-        if options.quality_cutoff is None:
-            options.quality_cutoff = "20,20"
-        if options.bisulfite not in ("non-directional", "truseq", "epignome", "swift"):
+        # TODO: set default adapter sequences
+        # Jury is out on whether quality trimming helps. For aligners like
+        # bwameth, it actually leads to worse results.
+        #if options.quality_cutoff is None:
+        #    options.quality_cutoff = "20,20"
+        if options.bisulfite == "swift" and paired != "both":
+            parser.error("Swift trimming is only compatible with paired-end reads")
+        if options.bisulfite not in ("rrbs", "non-directional", "truseq", "epignome", "swift"):
             def parse_bisulfite_params(r):
                 try:
                     parts = [int(p) for p in r.split(",")]
@@ -603,8 +639,6 @@ def validate_options(options, parser):
             elif paired != "both" and len(temp) > 1:
                 parser.error("Too many bisulfite parameters for single-end reads")
             options.bisulfite = temp
-        elif not paired and options.bisulfite != "non-directional":
-            parser.error("Single-end bisulfite libraries must be set to 'non-directional'.")
     
     if options.quality_cutoff is not None:
         cutoffs = options.quality_cutoff.split(',')
@@ -700,26 +734,6 @@ def validate_options(options, parser):
     
     if options.quiet or options.output is None or options.output == "-":
         options.progress = None
-    elif options.progress == "bar":
-        try:
-            import progressbar
-        except:
-            parser.error("The python-progressbar library is required for --progress=bar")
-    
-    # TODO: once we switch to argparse, int_or_str can be passed
-    # as the argument type
-    def int_or_str(x):
-        if x is None or isinstance(x, int):
-            return x
-        elif isinstance(x, str):
-            x = x.upper()
-            for a, mag in MAGNITUDE.items():
-                x = x.replace(a, mag[1])
-            return int(x)
-        else:
-            raise Exception("Unsupported type {}".format(x))
-    
-    options.max_reads = int_or_str(options.max_reads)
     
     if options.threads is not None:
         threads = options.threads
@@ -789,10 +803,9 @@ def create_reader(options, paired, parser, counter_magnitude="M"):
     reader = BatchIterator(reader, batch_size, options.max_reads)
     
     # Wrap iterator in progress bar
-    if options.progress == "msg":
-        reader = ProgressMessageReader(reader, batch_size)
-    elif options.progress == "bar":
-        reader = create_progress_reader(reader, options.max_reads, counter_magnitude)
+    if options.progress:
+        from atropos.progress import create_progress_reader
+        reader = create_progress_reader(reader, options.progress, batch_size, options.max_reads, counter_magnitude)
     
     return (reader, qualities, qualfile is not None)
 
@@ -849,106 +862,6 @@ class BatchIterator(object):
         self.done = True
         self.reader.close()
 
-class ProgressMessageReader(object):
-    def __init__(self, iterable, batch_size, interval=1000000):
-        self.iterable = iterable
-        self.batch_size = batch_size
-        self.interval = interval
-        self.ctr = 0
-    
-    def __next__(self):
-        value = self.iterable.next()
-        if value:
-            self.ctr += value[0]
-            if self.ctr % self.interval < self.batch_size:
-                now = time.time()
-                logging.getLogger().info("Read {0} records in {1:.1f} seconds".format(self.ctr, now - self.start))
-        return value
-    
-    next = __next__
-    def __iter__(self):
-        self.start = time.time()
-        return self
-    
-    def close(self):
-        logging.getLogger().info("Read a total of {} records".format(self.ctr))
-        self.iterable.close()
-        
-def create_progress_reader(reader, max_reads=None, counter_magnitude="M"):
-    try:
-        return create_progressbar_reader(reader, max_reads, counter_magnitude)
-    except:
-        pass
-    
-    try:
-        return create_tqdm_reader(reader)
-    except:
-        pass
-    
-    logging.getLogger().warn("No progress bar library available")
-    return reader
-
-def create_progressbar_reader(reader, max_reads=None, counter_magnitude="M"):
-    import progressbar
-    import progressbar.widgets
-    import math
-
-    class ProgressBarReader(progressbar.ProgressBar):
-        def __init__(self, iterable, widgets, max_value=None):
-            super(ProgressBarReader, self).__init__(
-                widgets=widgets, max_value=max_value or progressbar.UnknownLength)
-            self._iterable = iterable
-            self.done = False
-        
-        def __next__(self):
-            try:
-                value = next(self._iterable)
-                if self.start_time is None:
-                    self.start()
-                self.update(self.value + value[0])
-                return value
-            except StopIteration:
-                self.close()
-                raise
-        
-        def close(self):
-            if not self.done:
-                self.finish()
-                self.done = True
-            if not self._iterable.done:
-                self._iterable.close()
-    
-    class MagCounter(progressbar.widgets.WidgetBase):
-        def __init__(self, magnitude):
-            suffix = ""
-            if magnitude is None:
-                div = 1.0
-            else:
-                div = float(MAGNITUDE[magnitude][0])
-                suffix = magnitude
-        
-            self._format = lambda val: "{:.1f} {}".format(val / div, suffix)
-    
-        def __call__(self, progress, data):
-            return self._format(data["value"])
-        
-    if max_reads:
-        reader = ProgressBarReader(reader, [
-            MagCounter(counter_magnitude), " Reads (", progressbar.Percentage(), ") ",
-            progressbar.Timer(), " ", progressbar.Bar(), progressbar.AdaptiveETA()
-        ], max_reads)
-    else:
-        reader = ProgressBarReader(reader, [
-            MagCounter(counter_magnitude), " Reads", progressbar.Timer(),
-            progressbar.AnimatedMarker()
-        ])
-    
-    return reader
-
-def create_tqdm_reader(reader):
-    import tqdm
-    return tqdm.tqdm(reader)
-
 def create_modifiers(options, paired, qualities, has_qual_file, parser):
     adapter_parser = AdapterParser(
         colorspace=options.colorspace,
@@ -987,20 +900,19 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
         for adapter in adapters1 + adapters2:
             adapter.enable_debug()
     
-    merger = None
-    if options.merge_overlapping:
-        merger = MergeOverlapping(
-            min_overlap=options.merge_min_overlap,
-            error_rate=options.error_rate)
-    
-    modifiers = Modifiers(paired, merger)
+    modifiers = Modifiers(paired)
     
     for op in options.op_order:
         if op == 'A' and (adapters1 or adapters2):
             # TODO: generalize this using some kind of factory class
             if options.aligner == 'insert':
+                aligner_args = dict(
+                    max_insert_mismatch_frac=options.insert_match_error_rate,
+                    min_adapter_match_frac=1.0 - options.insert_match_adapter_error_rate
+                )
                 modifiers.add_modifier(InsertAdapterCutter,
-                    adapter1=adapters1[0], adapter2=adapters2[0], action=options.action)
+                    adapter1=adapters1[0], adapter2=adapters2[0], action=options.action,
+                    mismatch_action=options.correct_mismatches, **aligner_args)
             else:
                 modifiers.add_modifier_pair(AdapterCutter,
                     dict(adapters=adapters1, times=options.times, action=options.action),
@@ -1022,13 +934,13 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
     
     if options.bisulfite:
         if isinstance(options.bisulfite, str):
-            if options.bisulfite == "non_directional":
-                modifiers.add_modifier(NonDirectionalBisulfiteTrimmer)
-            elif options.bisulfite == "epignome":
-                modifiers.add_modifier(EpiGnomeBisulfiteTrimmer)
-            elif options.bisulfite == "truseq":
-                # TODO: how to trim?
-                pass
+            if "non-directional" in options.bisulfite:
+                modifiers.add_modifier(NonDirectionalBisulfiteTrimmer,
+                    rrbs=options.bisulfite=="non-directional-rrbs")
+            elif options.bisulfite == "rrbs":
+                modifiers.add_modifier(RRBSTrimmer)
+            elif options.bisulfite in ("epignome", "truseq"):
+                modifiers.add_modifier(TruSeqBisulfiteTrimmer)
             elif options.bisulfite == "swift":
                 modifiers.add_modifier(SwiftBisulfiteTrimmer)
         else:
@@ -1064,6 +976,11 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
     if options.trim_primer:
         modifiers.add_modifier(PrimerTrimmer)
     
+    if options.merge_overlapping:
+        modifiers.add_modifier(MergeOverlapping,
+            min_overlap=options.merge_min_overlap,
+            error_rate=options.error_rate)
+    
     return (modifiers, len(adapters1) + len(adapters2))
 
 def create_filters(options, paired, min_affected):
@@ -1120,13 +1037,13 @@ def create_formatters(options, qualities, default_outfile):
     if not formatters.multiplexed:
         if output1 is not None:
             formatters.add_seq_formatter(NoFilter, output1, output2)
-            if output1 != "-":
+            if output1 != "-" and not options.no_writer_process:
                 force_create.append(output1)
                 if output2 is not None:
                     force_create.append(output2)
         elif not (options.discard_trimmed and options.untrimmed_output):
             formatters.add_seq_formatter(NoFilter, default_outfile)
-            if default_outfile != "-":
+            if default_outfile != "-" and not options.no_writer_process:
                 force_create.append(default_outfile)
     
     if not options.discard_untrimmed:
