@@ -144,18 +144,18 @@ def test_match_probability():
     i5 = 0.25 ** 5
     assert approx_equal(a.match_probability(k, n), i3 + i4 + i5, 0.0001)
 
-def test_seqpurge_insert_align():
+def test_insert_align():
     a1_seq = 'TTAGACATATGG'
     a2_seq = 'CAGTGGAGTATA'
     aligner = InsertAligner(a1_seq, a2_seq)
     r1 = 'AGTCGAGCCCATTGCAGACT' + a1_seq[0:10]
     r2 = 'AGTCTGCAATGGGCTCGACT' + a2_seq[0:10]
-    match1, match2, insert_match = aligner.match_insert(r1, r2)
+    insert_match, insert_match_len, match1, match2 = aligner.match_insert(r1, r2)
     assert match1.rstart == 20
     assert match1.length == 10
     assert match2.rstart == 20
     assert match2.length == 10
-    assert insert_match == 20
+    assert insert_match_len == 20
     
 def test_short_adapter_overlap():
     a1_seq = 'TTAGACATAT'
@@ -163,7 +163,7 @@ def test_short_adapter_overlap():
     seq1 = 'GACAGGCCGTTTGAATGTTGACGGGATGTT'
     seq2 = 'CATCCCGTCAACATTCAAACGGCCTGTCCA'
     aligner = InsertAligner(a1_seq, a2_seq)
-    match1, match2, insert_match = aligner.match_insert(seq1, seq2)
+    insert_match, insert_match_len, match1, match2 = aligner.match_insert(seq1, seq2)
     assert match1.rstart == 28
     assert match1.length == 2
     assert match2.rstart == 28
